#include <avr\io.h>
#include <avr\interrupt.h>
#include <stdlib.h>
#include <stdint.h>
#include "UART.h"

uint8_t  flag=0, count=0;
uint16_t value[3];

ISR(TIMER1_COMPA_vect)
{
	static uint8_t channel=0;
	static uint16_t chargestart=0;

	// Aufladen sollte in jedem Fall fertig sein -> alles auf Entladen 
	DDRC=0xFF;

	// Wert += Capturewert - Startwert
	value[channel] += ICR1 - chargestart;

	// nchster Kanal
	channel++;

	// alle Kanle fertig?
	if(channel==3)
	{
		// Anzahl der Messungen zhlen
		count++;
		channel=0;	// von Vorne

		// Nach 8 Messungen
		if (count==8)
		{
			count=0;
			flag=1;		// Flag zur Auswertung
		}
	}

	ADMUX=channel;	// Kanal einstellen
	
	DDRC &= ~(1<<channel);	// Aufladen beginnen
	chargestart=TCNT1;		// Zeitpunkt merken
}

int main()
{
	char string[8];

	// Test, LED an
	DDRB  = (1<<0);
	PORTB = (1<<0);

	// PORTC als Ausgang, low -> Alle C entladen
	DDRC=0xFF;
	PORTC=0;
	
	// UART initialisieren, Hallo sagen
	UART_init();
	UART_print("Sensortaste v0.1\n");
	
	// Timer1 Noise canceler an, Input Capture rising edge, Prescaler 1
	// CTC-Modus, Compareregister 8192
	TCCR1B = (1<<ICNC1)|(0<<ICES1)|(1<<CS10)|(1<<WGM12);
	OCR1A=(1<<13);

	// Timer1 Output-Compare-Interrupt enable
	TIMSK  = (1<<OCIE1A);

	// Analog-Comparator Bandgap als pos-In
	ACSR=(1<<ACBG)|(1<<ACIC);
	// ADMUX als neg-IN
	SFIOR=(1<<ACME);

	// AD-Wandler aus
	ADCSRA &= ~(1<<ADEN);
	// Kanal 0 vorwhlen
	ADMUX = 0;

	// Interrupts aktivieren
	sei();

	for(;;)
	{
		if(flag)
		{
			// Ausgaben
			itoa(value[0]/8,string,10);
			UART_print(string);
			UART_print("; ");
			itoa(value[1]/8,string,10);
			UART_print(string);
			UART_print("; ");
			itoa(value[2]/8,string,10);
			UART_print(string);
			UART_ret;

			// Werte zurcksetzen
			value[0]=0;
			value[1]=0;
			value[2]=0;

			// Flag zurcksetzen
			flag=0;
		}
	}
}
